package ipl;

import diffcad.point;

public class Mat3 {
    double a[] [];

    private static double piOn180 = Math.PI / 180.0;

    public static Mat3 translation(point t) {
        double m[][] = new double[3][3];
        m[0][0] = 1;
        m[1][1] = 1;
        m[2][2] = 1;
        m[0][2] = t.x;
        m[1][2] = t.y;
        return new Mat3(m);
    }

    public static Mat3 scaling(point s) {
        double m[][] = new double[3][3];
        m[0][0] = s.x;
        m[1][1] = s.y;
        m[2][2] = 1;
        return new Mat3(m);
    }

    public static Mat3 shear(point sh) {
        double m[][] = new double[3][3];
        m[0][0] = 1;
        m[1][1] = 1;
        m[2][2] = 1;
        m[0][1] = sh.x;
        m[1][0] = sh.y;
        return new Mat3(m);
    }

    public static Mat3 rotation(double theta) {
        double m[][] = new double[3][3];
        double cas = Math.cos(theta);
        double sas = Math.sin(theta);
        m[0][0] = cas;
        m[1][1] = cas;
        m[0][1] = -sas;
        m[1][0] = sas;
        return new Mat3(m);
    }

    public static void main(String args[]) {
        Mat3 trans1 = Mat3.shear(new point(1, 2));
        trans1.print();
    }

    public Mat3(double a_[][]) {
        a = a_;
    }

    public double[][] getArray() {
        return a;
    }

/*
To generate a time-optimal 3x3 matrix inversion, use maple:
with(linalg):readlib(C):
a:=array(0..2,0..2,[]):
 b:=array(0..2,0..2,[]):
 b:=inverse(matrix(a)):
 C(b,optimized);

*/
    public Mat3 invert() {
        double b[] [] = new double[3][3];
        double t4 = a[0][0] * a[1][1];
        double t6 = a[0][0] * a[1][2];
        double t8 = a[0][1] * a[1][0];
        double t10 = a[0][2] * a[1][0];
        double t12 = a[0][1] * a[2][0];
        double t14 = a[0][2] * a[2][0];
        double t17 =
                1 / (-t4 * a[2][2] + t6 * a[2][1] + t8 * a[2][2] - t10 * a[2][1] - t12 * a[1][2] + t14 * a
                [1][1]);
        b[0][0] = -(a[1][1] * a[2][2] - a[1][2] * a[2][1]) * t17;
        b[0][1] = -(-a[0][1] * a[2][2] + a[0][2] * a[2][1]) * t17;
        b[0][2] = (-a[0][1] * a[1][2] + a[0][2] * a[1][1]) * t17;
        b[1][0] = (a[1][0] * a[2][2] - a[1][2] * a[2][0]) * t17;
        b[1][1] = (-a[0][0] * a[2][2] + t14) * t17;
        b[1][2] = -(-t6 + t10) * t17;
        b[2][0] = (-a[1][0] * a[2][1] + a[1][1] * a[2][0]) * t17;
        b[2][1] = -(-a[0][0] * a[2][1] + t12) * t17;
        b[2][2] = (-t4 + t8) * t17;

        return new Mat3(b);
    }
/* To generate a new matrix, transposed:
 with(linalg):readlib(C):
 a:=array(0..2,0..2,[]):
 b:=array(0..2,0..2,[]):
 b:=transpose(matrix(a)):
 C(b,optimized);
*/
    public Mat3 transpose() {
        double b[][] = new double[3][3];
        b[0][0] = a[0][0];
        b[0][1] = a[1][0];
        b[0][2] = a[2][0];
        b[1][0] = a[0][1];
        b[1][1] = a[1][1];
        b[1][2] = a[2][1];
        b[2][0] = a[0][2];
        b[2][1] = a[1][2];
        b[2][2] = a[2][2];
        return new Mat3(b);
    }


    public Mat3 multiply(Mat3 bmat3) {
        double WW [][] = new double[3][3];
        double b [][] = bmat3.getArray();
        WW[0][0] = a[0][0] * b[0][0] + a[0][1] * b[1][0] + a[0][2] * b[2][0];
        WW[0][1] = a[0][0] * b[0][1] + a[0][1] * b[1][1] + a[0][2] * b[2][1];
        WW[0][2] = a[0][0] * b[0][2] + a[0][1] * b[1][2] + a[0][2] * b[2][2];
        WW[1][0] = a[1][0] * b[0][0] + a[1][1] * b[1][0] + a[1][2] * b[2][0];
        WW[1][1] = a[1][0] * b[0][1] + a[1][1] * b[1][1] + a[1][2] * b[2][1];
        WW[1][2] = a[1][0] * b[0][2] + a[1][1] * b[1][2] + a[1][2] * b[2][2];
        WW[2][0] = a[2][0] * b[0][0] + a[2][1] * b[1][0] + a[2][2] * b[2][0];
        WW[2][1] = a[2][0] * b[0][1] + a[2][1] * b[1][1] + a[2][2] * b[2][1];
        WW[2][2] = a[2][0] * b[0][2] + a[2][1] * b[1][2] + a[2][2] * b[2][2];
        return (new Mat3(WW));
    }

/*
In maple:
vec3:=vector([v1, v2, v3]):
C(multiply(vec3,matrix(a)),optimized):

does a post-multiplication, A*v
*/
    public double[] multiply(double v1, double v2, double v3) {
        double x[] = new double[3];
        x[0] = v1 * a[0][0] + v2 * a[1][0] + v3 * a[2][0];
        x[1] = v1 * a[0][1] + v2 * a[1][1] + v3 * a[2][1];
        x[2] = v1 * a[0][2] + v2 * a[1][2] + v3 * a[2][2];
        return x;
    }
/*
To do a pre-multiplication, v*A
vec3:=vector([v1, v2, v3]):
> C(multiply(matrix(a),vec3),optimized):
*/
    public double[] premultiply(double v1, double v2, double v3) {
        double x[] = new double[3];
        x[0] = v1 * a[0][0] + a[0][1] * v2 + a[0][2] * v3;
        x[1] = a[1][0] * v1 + v2 * a[1][1] + a[1][2] * v3;
        x[2] = a[2][0] * v1 + a[2][1] * v2 + v3 * a[2][2];
        return x;
    }

    public void print() {
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++)
                System.out.print(a[i][j] + " ");
            System.out.println();
        }
    }

}