/**
 * WritePPM is a class that takes an Image and saves it to
 * a PPM format file.
 *
 * Victor Silva (victor@cse.bridgeport.edu).
 *
 */
package vs;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

public class WritePPM {
    int width;
    int height;

    public static void DoIt(short[] r, short[] g, short[] b, int imageWidth, int imageHeight,
                            String fname) throws Exception {
        WritePPM encode = new WritePPM(imageWidth, imageHeight);

        // Check for errors
        try {
            OutputStream output = new BufferedOutputStream(
                    new FileOutputStream(fname));
            encode.header(output);
            encode.image(output, r, g, b);
        } catch (Exception e) {
            System.out.println("Save PPM Exception - 2!");
        }
    }

    public WritePPM(int imageWidth, int imageHeight) {
        width = imageWidth;
        height = imageHeight;
    }


    void header(OutputStream out) throws IOException {
        writeString(out, "P6\n");
        writeString(out, width + " " + height + "\n");

        // Write the maximum value of each color.
        // i.e. r,g,b all vary between 0 and 255.
        writeString(out, "255\n");
    }

    static void writeString(OutputStream out, String str) throws IOException {
        int len = str.length();
        byte[] buf = new byte[len];
        str.getBytes(0, len, buf, 0);
        out.write(buf);
    }

    void image(OutputStream out, short[] r, short[] g, short[] b)
            throws IOException {
        int j;
        int i = 0;
        byte[] ppmPixels = new byte[width * 3];

        for (int row = 0; row < height; ++row) {
            j = 0;
            for (int col = 0; col < width; ++col) {
                ppmPixels[j++] = (byte) r[i];
                ppmPixels[j++] = (byte) g[i];
                ppmPixels[j++] = (byte) b[i++];
            }
            // Write ppm file a row at a time.
            out.write(ppmPixels);
        }
    }
}
